STOP_WORDS = set(
    """
ಹಲವು
ಮೂಲಕ
ಹಾಗೂ
ಅದು
ನೀಡಿದ್ದಾರೆ
ಯಾವ
ಎಂದರು
ಅವರು
ಈಗ
ಎಂಬ
ಹಾಗಾಗಿ
ಅಷ್ಟೇ
ನಾವು
ಇದೇ
ಹೇಳಿ
ತಮ್ಮ
ಹೀಗೆ
ನಮ್ಮ
ಬೇರೆ
ನೀಡಿದರು
ಮತ್ತೆ
ಇದು
ಈ
ನೀವು
ನಾನು
ಇತ್ತು
ಎಲ್ಲಾ
ಯಾವುದೇ
ನಡೆದ
ಅದನ್ನು
ಎಂದರೆ
ನೀಡಿದೆ
ಹೀಗಾಗಿ
ಜೊತೆಗೆ
ಇದರಿಂದ
ನನಗೆ
ಅಲ್ಲದೆ
ಎಷ್ಟು
ಇದರ
ಇಲ್ಲ
ಕಳೆದ
ತುಂಬಾ
ಈಗಾಗಲೇ
ಮಾಡಿ
ಅದಕ್ಕೆ
ಬಗ್ಗೆ
ಅವರ
ಇದನ್ನು
ಆ
ಇದೆ
ಹೆಚ್ಚು
ಇನ್ನು
ಎಲ್ಲ
ಇರುವ
ಅವರಿಗೆ
ನಿಮ್ಮ
ಏನು
ಕೂಡ
ಇಲ್ಲಿ
ನನ್ನನ್ನು
ಕೆಲವು
ಮಾತ್ರ
ಬಳಿಕ
ಅಂತ
ತನ್ನ
ಆಗ
ಅಥವಾ
ಅಲ್ಲ
ಕೇವಲ
ಆದರೆ
ಮತ್ತು
ಇನ್ನೂ
ಅದೇ
ಆಗಿ
ಅವರನ್ನು
ಹೇಳಿದ್ದಾರೆ
ನಡೆದಿದೆ
ಇದಕ್ಕೆ
ಎಂಬುದು
ಎಂದು
ನನ್ನ
ಮೇಲೆ
""".split()
)
